

/* application for bootloader */

/* 只包含需要的头文件 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/boot.h>

#include "app.h"
#include "config.h"

/* *************************
 * device initialization
 * argument(s) : void
 * return : void
 * ************************/
void device_init(void)
{
    usart_init();

    timer_init();

#if LED_EN
    SET_LED_PORT_OUT;
#endif

#if TRIGGER_MODE == 0
    SET_TRI_PORT_IN;
#endif
}

/* ************************
 * tiemr1 CTC mode
 * 100ms interrupt 
 * 1 / (16M / 1024) = 64us
 * OCR1A = 100ms * 1000 / 64us = 1562.5
 * ***********************/
void timer_init(void)
{
    TCCR1B = _BV(WGM12)|_BV(CS12)|_BV(CS20);/* N = 1024 CTC */
    OCR1A = OCR_VALUE;
}

/* ************************
 * 设置串口参数
 *
 * ***********************/
void usart_init(void) 
{
    /*设置波特率*/
    UBRRH = UBRRH_VALUE;
    UBRRL = UBRRL_VALUE; 
#if U2X_
    UCSRA |= _BV(U2X);
#endif
    /*接收器与发送器使能*/
    UCSRB = _BV(RXEN)|_BV(TXEN);
    /*设置帧格式: 8 个数据位, 1个停止位*/
    UCSRC = _BV(URSEL)|_BV(UCSZ1)|_BV(UCSZ0);
}

/* ************************
 * 发送单个字节至串口
 *
 * ***********************/
void write_udr(uint8_t c)
{
    //while(!CHKBIT(UCSRA, UDRE));
    /* 使用这种写法时收到EOT信号后
     * 回应ACK时至少要发送三次
     * write_udr(XMODEM_ACK);
     * 主机才能正常结束*/
    
    UDR = c;
    while(!CHKBIT(UCSRA, TXC));
#if U2X_
    UCSRA = _BV(TXC)|_BV(U2X);
#else
    UCSRA = _BV(TXC);
#endif
}

/* ************************
 * 接收单个字节
 *
 * ***********************/
uint8_t read_udr(void)
{
    while(!CHKBIT(UCSRA, RXC));
    return UDR;
}

/* ************************
 * 发送RAM字符串
 *
 * ***********************/
void put_string(char *str)
{
    while(*str)
	write_udr(*str++);
    write_udr(0x0d);
    write_udr(0x0a);
}

/* ***********************
 * program page
 * 页编程
 * 输入参数：page页地址 *buf数据
 * 返回参数：void
 * **********************/
void boot_program_page(uint32_t page, uint8_t *buf)
{
    uint16_t i;
    uint8_t sreg;

    // Disable interrupts.
    sreg = SREG;
    cli();

    eeprom_busy_wait();
    boot_page_erase(page);
    boot_spm_busy_wait(); // Wait until the memory is erased.
    
    for (i = 0; i < SPM_PAGESIZE; i += 2){
	
	// Set up little-endian word.
	uint16_t w = *buf++;
	w += (*buf++) << 8;

	boot_page_fill(page + i, w);
    }

    boot_page_write(page); // Store buffer in flash page.
    boot_spm_busy_wait();  // Wait until the memory is written.

    // Re-enable RWW-section again. We need this if we want to jump back
    // to the application after bootloading.
    //boot_rww_enable();

    //Re-enable interrupts (if they were ever enabled)

    SREG = sreg;
}

/* ***********************
 * 跳至用户程序
 *
 * **********************/
void quit(void)
{
#if 0
    DDRA = 0;
    DDRB = 0;
    DDRC = 0;
    DDRD = 0;
    PORTA = 0;
    PORTB = 0;
    PORTC = 0;
    PORTD = 0;
#endif

    boot_rww_enable();
    __asm__ __volatile__("jmp 0" : :);
    //(*((void(*)(void))0))();
}

/********** end of file ***********/
