
/* *************************
 * bootloader for charger
 * mowin
 * 2008-06-15
 * ************************/

/* 只包含需要的头文件 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>

#include "config.h"
#include "app.h"

/* XMODEM 数据帧 */
#define DATA_LEN 128
struct xmodem_pack
{
    uint8_t pack_no1;
    uint8_t pack_no2;
    uint8_t data[DATA_LEN];
    uint8_t crch;
    uint8_t crcl;
};
#define XMODEM_FRAME_LEN (sizeof(struct xmodem_pack))
#if DATA_LEN != SPM_PAGESIZE
#error XMODEM data len != SPM pagesize!
#endif

#define CRC_EN 1 /* 1 启用CRC 0 禁用CRC */

#if CRC_EN
/* 计算CRC校验(1021) */
uint8_t ch, cl;
void crc16(uint8_t *buf, uint8_t n)
{
    uint8_t j;
    uint8_t i;
    uint16_t crc, t;

    crc = 0;
    for(j = n; j > 0; j--){
	crc = (crc ^ (((uint16_t) *buf) << 8));
	for(i = 8; i > 0; i--){
	    t = crc << 1;
	    if(crc & 0x8000)
		t = t ^ 0x1021;
	    crc = t;
	}
	buf++;
    }
    ch = crc / 256;
    cl = crc % 256;
}
#endif


/* ***********************
 * function : main
 * argument(s) : void
 * return: 0=success 1=fail
 * **********************/
int main(void)
{
#define TRUE 0
#define FALSE -1
    uint16_t flash_addr;

    /* XMODEM data frame */
    struct xmodem_pack *rx_pack = NULL;
    /* receive buffer */
    uint8_t *buf = NULL;
    /* pack number */
    uint8_t pack_no;

    /* counter */
    uint8_t cnt;
    /* loop counter */
    uint8_t i;

    cli();
    device_init();

#if TRIGGER_MODE == 1
    /* 串口触发 */
    /* waiting for connected code 'd' */
    cnt = WAIT_C_CNT;
    while(1){

	if(CHKBIT(TIFR, OCF1A)){
	    TIFR = _BV(OCF1A);/* write 1 clear flag */

#if LED_EN
	    LED_ACT;
#endif
	    if(--cnt == 0)
		quit();
	}

	if(CHECK_RXC)
	    if(UDR == 'd')
		break; /* receive connected code */
    }
#elif TRIGGER_MODE == 0
    /* 
     * 端口触发
     * TRI_PIN = 0 --> bootloader
     * TRI_PIN = 1 --> normal boot 
     */
    cnt = WAIT_C_CNT;
    while(CHECK_TRI_PIN){

	if(CHKBIT(TIFR, OCF1A)){
	    TIFR = _BV(OCF1A);/* write 1 clear flag */

#if LED_EN
	    LED_ACT;
#endif
	    if(--cnt == 0)
		quit();
	}
    }
#else
#endif

    /* transmit XMODEM_RWC and wait for XMODEM_SOH */
    cnt = WAIT_D_CNT;
    i = 0;
    while(1){

	/* 收到XMODEM_SOH后开始接收数据 */
	if(CHECK_RXC){
	    if(UDR == XMODEM_SOH){
		break; /* start */
	    }
	}

	if(CHKBIT(TIFR, OCF1A)){
	    /* 100MS */
	    TIFR = _BV(OCF1A);/* write 1 clear flag */

	    /* 发送XMODEM＿RWC */
	    write_udr(XMODEM_RWC);
#if LED_EN
	    LED_ACT;
#endif

	    /* 10秒未收到XMODEM_SOH退出至用户程序 */
	    if(--cnt == 0){
		quit();
	    }
	}
    }

    TCCR1B = 0;/* turn off timer */

    /* start receive data frame */
    flash_addr = 0;
    pack_no = 1; /* 数据包序列号 */
    cnt = 0;

    /* 获取缓存空间 */
    buf = malloc(XMODEM_FRAME_LEN);
    if(buf == NULL)
	return FALSE;

    do{

	/* 读入一帧数据 */
	for(i = 0; i < XMODEM_FRAME_LEN; i++)
	    //buf[i] = read_udr();
	    *(buf+i) = read_udr();

	rx_pack = (struct xmodem_pack *)buf;

	rx_pack->pack_no2 = ~rx_pack->pack_no2;
	/* 检验包序列号 */
	if(rx_pack->pack_no1 == pack_no && rx_pack->pack_no2 == pack_no){

#if CRC_EN
	    /* 校验 CRC */
	    crc16(rx_pack->data, DATA_LEN);
	    if(rx_pack->crch == ch && rx_pack->crcl == cl){
#endif

		/* 数据正确 */
		boot_program_page(flash_addr, rx_pack->data);
		flash_addr += SPM_PAGESIZE;/* 下一页FLASH */

		/* 下一包 */
		pack_no++;
		write_udr(XMODEM_ACK);
		cnt = 0;
#if LED_EN
		LED_ACT;/* green LED */
#endif

#if CRC_EN
	    } else {
		/* CRC校验错重发 */
		write_udr(XMODEM_NAK);
		cnt++;
		//LED2_ACT;/* red LED */
	    }
#endif

	} else {
	    /* 包序列号错重发*/
	    write_udr(XMODEM_NAK);
	    cnt++;
	    //LED2_ACT;
	}

	/* 误码太多放弃升级 */
	//if(cnt > 10)
	//   break;


    } while(read_udr() != XMODEM_EOT);
    write_udr(XMODEM_ACK);

    free(buf);
    quit();
    return TRUE;
}
/* end of file */
